package com.christopherdro.RNPrint;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.print.PrintAttributes;
import android.print.PrintDocumentAdapter;
import android.print.PrintJob;
import android.print.PrintJobInfo;
import android.print.PrintManager;
import android.util.Log;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;

import androidx.core.content.FileProvider;

import com.facebook.common.file.FileUtils;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.UiThreadUtil;

import java.io.File;

/**
 * NativeModule that allows JS to open emails sending apps chooser.
 */
public class RNPrintModule extends ReactContextBaseJavaModule implements RNPrintListener {

    private ReactApplicationContext reactContext;
    private WebView mWebView;
    private Promise activePromise = null;
    private PrintJob currentJob;

    public RNPrintModule(ReactApplicationContext reactContext) {
        super(reactContext);
        this.reactContext = reactContext;
    }

    @Override
    public String getName() {
        return "RNPrint";
    }

    @ReactMethod
    public void print(final ReadableMap options, final Promise promise) {
        if(activePromise != null) {
            promise.reject(new Error("Awaiting another promise"));
        } else {
            activePromise = promise;
            final String html = options.hasKey("html") ? options.getString("html") : null;
            final String filePath = options.hasKey("filePath") ? options.getString("filePath") : null;
            final String fileName = options.hasKey("fileName") ? options.getString("fileName") : "EpaperArticle";
            final boolean isLandscape = options.hasKey("isLandscape") ? options.getBoolean("isLandscape") : false;
            final boolean printToLetter = options.hasKey("printToLetter") ? options.getBoolean("printToLetter") : true;

            if ((html == null && filePath == null) || (html != null && filePath != null)) {
                reject( "Must provide either `html` or `filePath`.  Both are either missing or passed together");
                return;
            }

            Activity currentActivity = getCurrentActivity();
            if (currentActivity == null) {
                reject("Activity is null");
                return;
            }

            final Context context = currentActivity;

            if (html != null) {
                UiThreadUtil.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        // Create a WebView object specifically for printing
                        WebView webView = new WebView(reactContext);
                        webView.getSettings().setAllowFileAccess(true);
                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                            webView.getSettings().setMixedContentMode(WebSettings.MIXED_CONTENT_ALWAYS_ALLOW);
                        }
                        webView.setWebViewClient(new WebViewClient() {

                            public boolean shouldOverrideUrlLoading(WebView view, String url) {
                                return false;
                            }

                            @Override
                            public void onPageFinished(WebView view, String url) {
                                try {
                                    createWebPrintJob(context, view, fileName, printToLetter, isLandscape);
                                } catch (Exception e) {
                                    reject(e.getMessage());
                                }
                                mWebView = null;
                            }
                        });

                        webView.loadDataWithBaseURL(null, html, "text/HTML", "UTF-8", null);
                        mWebView = webView;
                    }
                });
            } else {
                try {
                    //sharePdf(context, filePath, fileName);
                    printPdf(context, filePath, fileName, printToLetter, isLandscape);
                    resolve(fileName);
                } catch (Exception e) {
                    e.printStackTrace();
                    reject(e.getMessage());
                }
            }
        }
    }

    private void reject(String message){
        if(activePromise != null){
            activePromise.reject(getName(), message);
        }
        activePromise = null;
    }

    private void resolve(String fileName){
        if(activePromise != null){
            activePromise.resolve(fileName);
        }
        activePromise = null;
    }

    private PrintAttributes getAttributes(boolean printToLetter, boolean isLandscape){
        PrintAttributes.Builder builder = new PrintAttributes.Builder();
        PrintAttributes.MediaSize size = printToLetter ? PrintAttributes.MediaSize.NA_LETTER : PrintAttributes.MediaSize.ISO_A4;
        builder.setMediaSize(isLandscape ? size.asLandscape() : size.asPortrait());
        builder.setMinMargins(PrintAttributes.Margins.NO_MARGINS);
        return builder.build();
    }

    private void printPdf(Context context, String filePath, String jobName, boolean printToLetter, boolean isLandscape){
        File file = new File(context.getCacheDir(), new File(filePath).getName());
        PrintDocumentAdapter printAdapter = new PdfDocumentAdapter( context, file.getPath(), jobName );
        print( context, new RNPrintAdapterWrapper(printAdapter, RNPrintModule.this), getAttributes(printToLetter, isLandscape), jobName);
    }

    private void createWebPrintJob(Context context, WebView webView, final String fileName, boolean printToLetter, boolean isLandscape) {
        final PrintDocumentAdapter printAdapter;
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP) {
            printAdapter = webView.createPrintDocumentAdapter(fileName);
        } else {
            printAdapter = webView.createPrintDocumentAdapter();
        }
        print(context, printAdapter, getAttributes(printToLetter, isLandscape), fileName);
    }

    private void print(Context context, PrintDocumentAdapter printAdapter, PrintAttributes attributes, String name ){
        PrintManager printManager = (PrintManager) context.getSystemService(Context.PRINT_SERVICE);
        currentJob = printManager.print(name, new RNPrintAdapterWrapper(printAdapter, RNPrintModule.this), attributes);
    }

    @Override
    public void onStart() {
        Log.d(getName(), "Print start a Job");
    }

    @Override
    public void onFinish() {
        try {
            int state = currentJob.getInfo().getState();
            Log.d(getName(), "Print state for Job: " + state);
            if(state == PrintJobInfo.STATE_STARTED || state == PrintJobInfo.STATE_COMPLETED){
                resolve(currentJob.getInfo().getLabel());
            } else {
                //PrintJobInfo.STATE_CREATED, PrintJobInfo.STATE_QUEUED, PrintJobInfo.STATE_BLOCKED, PrintJobInfo.STATE_FAILED, PrintJobInfo.STATE_CANCELED,
                resolve("");
            }
        } catch(Exception e){
            reject(e.getMessage());
        }
    }

    private void sharePdf(Context context, String filePath, String jobName) throws FileUtils.RenameException {
        File file = new File(context.getCacheDir(), new File(filePath).getName());
        File printsPath = new File(context.getCacheDir(), "prints");
        printsPath.mkdirs();
        File newFile = new File(printsPath, jobName.replaceAll("\\.", "") + ".pdf");
        FileUtils.rename(file,newFile);

        Uri uri = FileProvider.getUriForFile(context, context.getPackageName()+".print.fileprovider", newFile);
        Intent intent = new Intent();
        intent.setDataAndType(uri, "application/pdf");
        intent.setAction(Intent.ACTION_VIEW);
        intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
        getCurrentActivity().startActivity(intent);
    }
}
